/**
 * @file Ce fichier génère le schéma GraphQL. C'est ici que les requêtes GraphQl sont résolues.
 * @author akka vodol
*/
import { makeExecutableSchema } from 'graphql-tools';
import { request } from 'https';
import { assertBinaryExpression } from 'babel-types';

import typeDefs from './typeDefs';

import * as resolverFuncs from './db_utils';

/** 
 * @description Résolveurs des différentes requêtes GraphQL
*/
const resolvers = {
    Query: {

        asAdmin: (obj, args, context) => {
            return resolverFuncs.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have admin rights over this group";
            });
        },

        allGroups: (obj, args, context) => {
            return resolverFuncs.getAllVisibleGroups(context.user);
        },
        group: (obj, args, context) => {
            return resolverFuncs.getGroupIfVisible(context.user, args.uid);
        },
        user: (obj, args, context) => {
            const refactorer = (data) => {
                return {
                    uid: args.uid,
                    lastName: data.sn,
                    givenName: data.givenName,
                    birthdate: data.brBirthdate,
                    groups: data.brMemberOf,
                    mail: data.mail,
                    phone: data.telephoneNumber,
                    room: data.brRoom
                };
            };

            console.log("Fetching user data as user",context.user);

            const result = resolverFuncs.renseignerSurUtilisateur(context.user, args.uid).then(res => {
                const output = res.map(entry => refactorer(entry));
                return output;
            });

            return result;
        },

        searchTOL: (obj, args, context) => {
            console.log(args);
            return resolverFuncs.repliquerTOLdesIds(context.user, {
                givenName: args.givenName,
                lastName: args.lastName,
                nickname: args.nickname,
                nationality: args.nationality,
                school: args.school,
                promotion: args.promotion,
                groups: args.groups,
                studies: args.studies,
                sport: args.sport,
                phone: args.phone,
                mail: args.mail,
                adress: args.adress,
                ip: args.ip
            });
        }
    },

    Mutation: {
        asAdmin: (obj, args, context) => {
            return resolverFuncs.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have admin rights over this group";
            });
        },

        asSpeaker: (obj, args, context) => {
            return resolverFuncs.hasAdminRights(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You do not have publishing rights in this group";
            });
        },

        asMember: (obj, args, context) => {
            return resolverFuncs.isMember(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You are  not a member of this group";
            });
        },

        asViewer: (obj, args, context) => {
            return resolverFuncs.getGroupIfVisible(context.user, args.groupUID).then(res => {
                if(res)
                    return {groupUID : args.groupUID};
                else
                    throw "You are  not a member of this group";
            });
        }
    },

    AdminMutation: {
    
        isAdmin: (obj, args, context) => {
            return true;
        
        },
        createSubgroup: (obj, args, context) => {
            args.parentuid = obj.groupUID;
            return resolverFuncs.createSubgroup(context.user, args);
        },

        editGroup: (obj, args, context) => {
            return null;
        }
    },

    ViewerMutation: {

        requestJoin: (obj, args, context) => {
            console.log(obj);
            return null;
        }
    }

};

const schema = makeExecutableSchema({
    typeDefs,
    resolvers
});

export default schema;